import socket
import threading
import tkinter as tk
from tkinter import scrolledtext, messagebox



# Global client socket variable
client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

# Create a stop_event to control the receiving thread
stop_event = threading.Event()

# Function to handle receiving messages
def receive_messages():
    while not stop_event.is_set():
        try:
            message = client_socket.recv(1024).decode('utf-8').strip()

            if not message:
                break

            # Update the chat display with received message
            chat_display.config(state=tk.NORMAL)  # Enable text widget to insert text
            chat_display.insert(tk.END, f"Server: {message}\n")
            chat_display.yview(tk.END)
            chat_display.config(state=tk.DISABLED)  # Disable text widget after updating
        except Exception as e:
            print(f"Error receiving message: {e}")
            break

# Start the thread that receives messages
def start_receiving_thread():
    receive_thread = threading.Thread(target=receive_messages, daemon=True)
    receive_thread.start()

# Function to send messages
def send_message():
    msg = entry_message.get()
    if msg:
        try:
            client_socket.send(msg.encode('utf-8'))
            # Update the chat display with sent message
            chat_display.config(state=tk.NORMAL)
            chat_display.insert(tk.END, f"You: {msg}\n")
            chat_display.yview(tk.END)
            chat_display.config(state=tk.DISABLED)
        except Exception as e:
            print(f"Error sending message: {e}")
    entry_message.delete(0, tk.END)  # Clear the message input field

# Function to handle login and joining the server
def login():
    username = entry_username.get()
    password = entry_password.get()
    server_name = entry_server_name.get()

    if not username or not password:
        messagebox.showwarning("Input Error", "Please enter both username and password.")
        return

    # Send the login command to server
    login_command = f"/login {username} {password}"
    try:
        client_socket.send(login_command.encode('utf-8'))
    except Exception as e:
        print(f"Error sending login command: {e}")

    # Send the /join_server command
    if server_name:
        join_command = f"/join_server {server_name}"
    else:
        join_command = "/join_server general"  # Default to "general" if no server is specified

    try:
        client_socket.send(join_command.encode('utf-8'))
    except Exception as e:
        print(f"Error sending /join_server command: {e}")

    # Close the login window and move to the main chat window
    login_window.destroy()
    start_receiving_thread()
    chat_window.deiconify()  # Show the chat window after successful login

# Set up the client socket and connect to the server
def connect_to_server():
    address = entry_address.get() if custom_address.get() else "147.185.221.19"
    port = int(entry_port.get()) if custom_address.get() else 42439
    
    try:
        client_socket.connect((address, port))  # Connect using the provided address and port
        messagebox.showinfo("Connection", f"Connected to {address}:{port}")
    except Exception as e:
        messagebox.showerror("Connection Error", f"Unable to connect to the server: {e}")
        root.quit()

# Function to send a private message
def send_private_message():
    sender = entry_sender.get()  # Get sender name
    recipient = entry_recipient.get()
    private_message = entry_private_message.get()

    if not recipient or not private_message:
        messagebox.showwarning("Input Error", "Please enter both recipient and message.")
        return

    # Send /pm command with sender, recipient, and message
    pm_command = f"/pm {recipient} {private_message}"
    try:
        client_socket.send(pm_command.encode('utf-8'))
        # Update the chat display with the sent private message
        chat_display.config(state=tk.NORMAL)
        chat_display.insert(tk.END, f"You (private to {recipient}): {private_message}\n")
        chat_display.yview(tk.END)
        chat_display.config(state=tk.DISABLED)
    except Exception as e:
        print(f"Error sending private message: {e}")
    
    private_message_window.destroy()  # Close the private message window after sending

# Open the private message window
def open_private_message_window():
    global private_message_window, entry_sender, entry_recipient, entry_private_message
    
    private_message_window = tk.Toplevel(chat_window)
    private_message_window.title("Send Private Message")
    private_message_window.configure(bg='lightblue')
    

    # Create the sender label and entry field
    label_sender = tk.Label(private_message_window, text="Sender (Optional):", fg="black", bg="lightblue")
    label_sender.grid(row=0, column=0, padx=10, pady=10)
    entry_sender = tk.Entry(private_message_window, bg="lightblue", fg="black")
    entry_sender.grid(row=0, column=1, padx=10, pady=10)

    # Create the recipient label and entry field
    label_recipient = tk.Label(private_message_window, text="Recipient:", fg="black", bg="lightblue")
    label_recipient.grid(row=1, column=0, padx=10, pady=10)
    entry_recipient = tk.Entry(private_message_window, bg="lightblue", fg="black")
    entry_recipient.grid(row=1, column=1, padx=10, pady=10)

    # Create the private message label and entry field
    label_private_message = tk.Label(private_message_window, text="Message:", fg="black", bg="lightblue")
    label_private_message.grid(row=2, column=0, padx=10, pady=10)
    entry_private_message = tk.Entry(private_message_window, bg="lightblue")
    entry_private_message.grid(row=2, column=1, padx=10, pady=10)

    # Create the send button
    send_private_button = tk.Button(private_message_window, text="Send Private Message", bg="lightgreen", fg="black",  command=send_private_message)
    send_private_button.grid(row=3, column=0, columnspan=2, pady=10)

# Create the main GUI window (chat window) and hide it initially
root = tk.Tk()
root.title("Socket Chat Client", )
root.configure(bg='lightblue')
# Create a hidden chat window that will appear after successful login
chat_window = tk.Toplevel(root)
chat_window.title("Chat Window")
chat_window.withdraw()  # Hide the chat window initially
chat_window.configure(bg='lightblue')

# Create a chat display area
chat_display = scrolledtext.ScrolledText(chat_window, wrap=tk.WORD, width=50, height=15, state=tk.DISABLED, bg="lightblue", fg="black")
chat_display.grid(row=0, column=0, padx=10, pady=10)

# Create the message entry field
entry_message = tk.Entry(chat_window, width=40, bg="lightblue", fg="black",)
entry_message.grid(row=1, column=0, padx=10, pady=10)

# Create the send button
send_button = tk.Button(chat_window, text="Send", bg="lightgreen", fg="black", command=send_message)
send_button.grid(row=1, column=1, padx=10, pady=10)

# Create the private message button
private_message_button = tk.Button(chat_window, text="Send Private Message", bg="lightgreen", fg="black", command=open_private_message_window)
private_message_button.grid(row=2, column=0, columnspan=2, pady=10)

# Create the login window
login_window = tk.Toplevel(root)
login_window.title("Login")
login_window.configure(bg='lightblue')

# Create the username and password labels and entry fields
label_username = tk.Label(login_window, text="Username:", fg="black", bg="lightblue")
label_username.grid(row=0, column=0, padx=10, pady=10)
entry_username = tk.Entry(login_window, bg="lightblue", fg="black")
entry_username.grid(row=0, column=1, padx=10, pady=10)

label_password = tk.Label(login_window, text="Password:", fg="black", bg="lightblue")
label_password.grid(row=1, column=0, padx=10, pady=10)
entry_password = tk.Entry(login_window, show="*", bg="lightblue", fg="black")
entry_password.grid(row=1, column=1, padx=10, pady=10)

# Create the server name label and entry field
label_server_name = tk.Label(login_window, text="Server Name (Optional):", fg="black", bg="lightblue")
label_server_name.grid(row=2, column=0, padx=10, pady=10)
entry_server_name = tk.Entry(login_window, bg="lightblue", fg="black")
entry_server_name.grid(row=2, column=1, padx=10, pady=10)

# Create the login button
login_button = tk.Button(login_window, text="Login", bg="lightgreen", fg="black", command=login)
login_button.grid(row=3, column=0, columnspan=2, pady=10)

# Create the connection configuration window
config_window = tk.Toplevel(root)
config_window.title("Server Configuration")
config_window.configure(bg='lightblue')

# Checkbox to toggle between default or custom address
custom_address = tk.BooleanVar(value=False)

# Create the address input field and checkbox
label_address = tk.Label(config_window, text="Server Address:", fg="black", bg="lightblue")
label_address.grid(row=0, column=0, padx=10, pady=10)
entry_address = tk.Entry(config_window, fg="black", bg="lightblue")
entry_address.grid(row=0, column=1, padx=10, pady=10)
entry_address.insert(0, "147.185.221.19")

label_port = tk.Label(config_window, text="Port:", fg="black", bg="lightblue")
label_port.grid(row=1, column=0, padx=10, pady=10)
entry_port = tk.Entry(config_window, bg="lightblue", fg="black")
entry_port.grid(row=1, column=1, padx=10, pady=10)
entry_port.insert(0, "42439")

# Checkbox to choose custom address
checkbox_custom = tk.Checkbutton(config_window, text="Use Custom Address", bg="lightgreen", fg="black", variable=custom_address, command=lambda: toggle_custom_fields())
checkbox_custom.grid(row=2, column=0, columnspan=2, pady=10)

# Function to toggle visibility of custom address and port fields
def toggle_custom_fields():
    if custom_address.get():
        entry_address.config(state=tk.NORMAL)
        entry_port.config(state=tk.NORMAL)
    else:
        entry_address.config(state=tk.DISABLED)
        entry_port.config(state=tk.DISABLED)

# Button to connect to the server
connect_button = tk.Button(config_window, text="Connect", bg="lightgreen", fg="black", command=connect_to_server)
connect_button.grid(row=3, column=0, columnspan=2, pady=10)

# Start the GUI event loop
root.mainloop()

# Make sure to close the socket when exiting
client_socket.close()
